!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Calculates cylindrical coordinates (and curvatures) using FFT on regular grid:

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex $(\s,\t,\z) \mapsto (R,\phi,Z)$.
!latex This routine calculates, $R$ and $Z$, and their first and second derivatives wrt $\s$, $\t$, and $\z$.
!latex The metric elements are calculated, and their derivatives.
!latex Note, the second derivatives are only calculated if \verb+Lcurv.eq.1+ on input.
!latex All quantities are returned through global arrays, namely \verb+Rij,Zij+, \verb+gij+ . . .
!latex The jacobian, $\sqrt g$, and lower metric elements, $g_{ij}/\sqrt g$, are also returned through global arrays.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! given lss (radial coordinate -- must be in ll-th interval), returns Rij,Zij (coordinates and derivatives) and sg,gij on regular grid;

subroutine co01aa( lss , Lcurv )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half, two
  use numerical, only : sqrtmachprec
  use fileunits, only : ounit
  use inputlist, only : Wco01aa, Nvol, Ni, Icoords
  use cputiming, only : Tco01aa
  use allglobal, only : myid, cpus, mn, im, in, ss, Rbc, Zbs, Rmn, Zmn, ll, pi2nfp, iss, &
                        Nt, Nz, Ntz, isr, trigm, trign, trigwk, &
                        sg, gij, &
                        Rij, Zij, efmn, ofmn, cfmn, sfmn, ijreal, ijimag
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS

  REAL   , intent(in) :: lss   ! radial coordinate;
  INTEGER, intent(in) :: Lcurv ! flag indicating that "curvature" terms \equiv \partial_k g_{i,j} etc. to be calculated;
  
  INTEGER             :: imn, Nint, ie01bgf, ii, ij, ik, ie, if, ig
  REAL                :: llss(1), dRZmnu, dRZmnl
  
  BEGIN(co01aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  FATAL(co01aa,lss.lt.ss(ll)%s(0))      ! note that calculations in each annulus are performed separately;
  FATAL(co01aa,lss.gt.ss(ll)%s(Ni(ll))) ! present annulus is determined by global variable ll;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  llss(1) = lss

  Rij(0:9,1:Ntz)=zero ; Zij(0:9,1:Ntz)=zero ; sg(0:3,1:Ntz)=zero ; gij(1:6,0:3,1:Ntz)=zero ! initialize summation variables; these are returned through global;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item For the innermost volume, \verb+ll.eq.1+, an additional `analytic' term, $(s/s_1)^m$, is added to the coordinates.
!latex This make the derivatives incorrect! This is under construction.
   
 !FATAL(co01aa,ll.eq.1)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  do imn = 1,mn ! loop over Fourier modes; perform interpolation;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex \item For \verb+Icoords.eq.0+, the coordinate interpolation is a straight linear interpolation between the interfaces, eg.
!latex \be R_j = R_{l-1,j} + \left(R_{l,j}-R_{l-1,j}\right) \frac{ ( s - s_{l-1} ) }{ ( s_{l}-s_{l-1})}.
!latex \ee
   
   select case(Icoords)
    
   case(0) ! linear interpolation; derivatives calculated in ih00aa; NOTE : this may be over-written below;
    
    Rmn(0,imn) = Rbc(ll-1,0,imn) +  Rbc(ll,1,imn) * ( lss - ss(ll)%s(0) )
    Rmn(1,imn) =                    Rbc(ll,1,imn)                        
    Rmn(2,imn) =                    zero                                 
    Zmn(0,imn) = Zbs(ll-1,0,imn) +  Zbs(ll,1,imn) * ( lss - ss(ll)%s(0) )
    Zmn(1,imn) =                    Zbs(ll,1,imn)                        
    Zmn(2,imn) =                    zero                                 
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
!latex \item For \verb+Icoords.eq.1+, the coordinate interpolation is a piecewise cubic interpolation,
!latex provided by the NAG routine \verb+E01BGF+.
!latex Recall that the derivatives, \verb+Rbc(0:Nvol,1,1:mn), Zbs(1:Nvol,1,1:mn)+ have been calculated in \verb+ih00aa+.
    
   case(1:2) ! monotonic (NAG) / homespun cubic interpolation; derivatives calculated in ih00aa;
    
    Nint = 1
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

    Rmn(2,imn)=zero ! THIS IS INCORRECT; second derivatives need to be extracted from piecewise cubic;
    
    if( Lcurv.ge.2 ) then ! use finite-differences to estimate 2nd derivative; rather clumsy; should extract 2nd derivative exactly;
     
     ie01bgf=1 ; llss(1) = lss + sqrtmachprec
     call E01BGF( Nvol+1 , iss(0:Nvol) , Rbc(0:Nvol,0,imn) , Rbc(0:Nvol,1,imn) , Nint , llss , Rmn(0,imn) , dRZmnu ,ie01bgf )
     FATAL(co01aa,ie01bgf.ne.0)
     
     ie01bgf=1 ; llss(1) = lss - sqrtmachprec
     call E01BGF( Nvol+1 , iss(0:Nvol) , Rbc(0:Nvol,0,imn) , Rbc(0:Nvol,1,imn) , Nint , llss , Rmn(0,imn) , dRZmnl ,ie01bgf )
     FATAL(co01aa,ie01bgf.ne.0)
     
     Rmn(2,imn) = ( dRZmnu - dRZmnl ) * half / sqrtmachprec

    endif
    
    ie01bgf=1 ; llss(1) = lss
    call E01BGF( Nvol+1 , iss(0:Nvol) , Rbc(0:Nvol,0,imn) , Rbc(0:Nvol,1,imn) , Nint , llss , Rmn(0,imn) , Rmn(1,imn) ,ie01bgf )
    FATAL(co01aa,ie01bgf.ne.0)
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
    Zmn(2,imn)=zero ! THIS IS INCORRECT; second derivatives need to be extracted from piecewise cubic;
    
    if( Lcurv.ge.2 ) then ! use finite-differences to estimate 2nd derivative; rather clumsy; should extract 2nd derivative exactly;
     
     ie01bgf=1 ; llss(1) = lss + sqrtmachprec
     call E01BGF( Nvol+1 , iss(0:Nvol) , Zbs(0:Nvol,0,imn) , Zbs(0:Nvol,1,imn) , Nint , llss , Zmn(0,imn) , dRZmnu ,ie01bgf )
     FATAL(co01aa,ie01bgf.ne.0)

     ie01bgf=1 ; llss(1) = lss - sqrtmachprec
     call E01BGF( Nvol+1 , iss(0:Nvol) , Zbs(0:Nvol,0,imn) , Zbs(0:Nvol,1,imn) , Nint , llss , Zmn(0,imn) , dRZmnl ,ie01bgf )
     FATAL(co01aa,ie01bgf.ne.0)
     
     Zmn(2,imn) = ( dRZmnu - dRZmnl ) * half / sqrtmachprec

    endif

    ie01bgf=1 ; llss(1) = lss
    call E01BGF( Nvol+1 , iss(0:Nvol) , Zbs(0:Nvol,0,imn) , Zbs(0:Nvol,1,imn) , Nint , llss , Zmn(0,imn) , Zmn(1,imn) ,ie01bgf )
    FATAL(co01aa,ie01bgf.ne.0)
    
   case default
    
    FATAL(co01aa : invalid Icoords,.true.)
    
   end select
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! the interpolated Fourier harmonics are now constructed; now perform fft to reconstruct geometry and derivatives;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The cylindrical coordinates and metric elements are constructed on a regular grid, so FFT's are used. 
!latex The cylindrical coordinates (and derivatives) are returned 
!latex through the global arrays \verb+Rij(0:3,Nt*Nz)+,\verb+Zij(0:3,Nt*Nz)+.
  
  ofmn(1:mn)=zero ; cfmn(1:mn)=zero
  call invfft( mn , im , in , Rmn(0,:) , ofmn(1:mn) , cfmn(1:mn) , Zmn(0,:) , Nt , Nz , Rij(0,:) , Zij(0,:) , isr , trigm , trign , trigwk )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item if \verb+Lcurv.eq.0+, only the coordinates \verb+Rij(0,:),Zij(0,:)+ are constructed.
  if( Lcurv.le.0 ) goto 9999 ! the metric and the "curvature" terms are not needed
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  ofmn(1:mn)=zero ; cfmn(1:mn)=zero
  call invfft( mn , im , in ,Rmn(1,:) , ofmn(1:mn) , cfmn(1:mn) , Zmn(1,:) , Nt , Nz , Rij(1,:) , Zij(1,:) , isr , trigm , trign , trigwk )
  
  efmn(1:mn)=zero ; sfmn(1:mn)=zero
  call invfft( mn , im , in , efmn(1:mn) , -im*Rmn(0,:) , +im*Zmn(0,:) , sfmn(1:mn) , Nt , Nz , Rij(2,:) , Zij(2,:) , isr , trigm , trign , trigwk )
  
  efmn(1:mn)=zero ; sfmn(1:mn)=zero
  call invfft( mn , im , in , efmn(1:mn) , +in*Rmn(0,:) , -in*Zmn(0,1:mn) , sfmn(:) , Nt , Nz , Rij(3,:) , Zij(3,:) , isr , trigm , trign , trigwk )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The jacobian is \be \sqrt g = R ( R_s Z_\theta - R_\theta Z_s ). \ee
  sg(0,:) = Rij(0,:) * ( Rij(1,:)*Zij(2,:) - Rij(2,:)*Zij(1,:) )
  
 !cput = MPI_WTIME() ; write(ounit,'("co01aa : "f10.2" s : myid="i3" : sg \in ["es13.5","es13.5" ] ;")')cput-cpus,myid,minval(sg(0,1:Ntz)),maxval(sg(0,1:Ntz))
 !FATAL(co01aa,minval(sg(0,1:Ntz))*minval(sg(0,1:Ntz)).lt.zero)
  
!latex \item The `lower' metric coefficients, $g_{ij}$, are given by
!latex \be g_{\s\s} &=& R_{\s}R_{\s} + Z_{\s}Z_{\s},    \\
!latex     g_{\s\t} &=& R_{\s}R_{\t} + Z_{\s}Z_{\t},    \\
!latex     g_{\s\z} &=& R_{\s}R_{\z} + Z_{\s}Z_{\z},    \\
!latex     g_{\t\t} &=& R_{\t}R_{\t} + Z_{\t}Z_{\t},    \\
!latex     g_{\t\z} &=& R_{\t}R_{\z} + Z_{\t}Z_{\z},    \\
!latex     g_{\z\z} &=& R_{\z}R_{\z} + Z_{\z}Z_{\z}+R^2.
!latex \ee
  
!latex \item Note the following: \\
!latex \verb+gij(1,0,:)+$=g_{\s\s}$ ,
!latex \verb+gij(1,1,:)+$=\partial_\s g_{\s\s}$ ; 
!latex \verb+gij(1,2,:)+$=\partial_\t g_{\s\s}$ ; 
!latex \verb+gij(1,3,:)+$=\partial_\z g_{\s\s}$ \\
!latex \verb+gij(2,0,:)+$=g_{\s\t}$ \\
!latex \verb+gij(3,0,:)+$=g_{\s\z}$ \\
!latex \verb+gij(4,0,:)+$=g_{\s\s}$ \\
!latex \verb+gij(5,0,:)+$=g_{\s\t}$ \\
!latex \verb+gij(6,0,:)+$=g_{\s\z}$ \\
  
  gij(1,0,:) = Rij(1,:) * Rij(1,:)                       + Zij(1,:) * Zij(1,:) ! g_{s,s}
  gij(2,0,:) = Rij(1,:) * Rij(2,:)                       + Zij(1,:) * Zij(2,:) ! g_{s,t}
  gij(3,0,:) = Rij(1,:) * Rij(3,:)                       + Zij(1,:) * Zij(3,:) ! g_{s,z}
  gij(4,0,:) = Rij(2,:) * Rij(2,:)                       + Zij(2,:) * Zij(2,:) ! g_{t,t}
  gij(5,0,:) = Rij(2,:) * Rij(3,:)                       + Zij(2,:) * Zij(3,:) ! g_{t,z}
  gij(6,0,:) = Rij(3,:) * Rij(3,:) + Rij(0,:) * Rij(0,:) + Zij(3,:) * Zij(3,:) ! g_{z,z}
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item if \verb+Lcurv.eq.1+, the coordinates derivatives \verb+Rij(1:3,:),Zij(1:3,:)+, 
!latex the Jacobian \verb+sg(0,:)+, and the `lower' metric elements \verb+gij(1:6,0,:)+ are also constructed.
  
  if( Lcurv.le.1 ) goto 9999 ! the "curvature" terms are not needed
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item If the input flag \verb+Lcurv.eq.1+, then the "curvature" terms are calculated. 
!latex These terms are required to compute $\nabla \times {\bf B}$,
!latex so that the accuracy of $\nabla \times {\bf B} = \mu {\bf B}$ can be quantified.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Note the following: \\
!latex \verb+Rij(0,:)+$=R$ \\
!latex \verb+Rij(1,:)+$=R_\s$ \\ 
!latex \verb+Rij(2,:)+$=R_\t$ \\
!latex \verb+Rij(3,:)+$=R_\z$ \\
!latex \verb+Rij(4,:)+$=R_{\s\s}$ \\
!latex \verb+Rij(5,:)+$=R_{\s\t}$ \\
!latex \verb+Rij(6,:)+$=R_{\s\z}$ \\
!latex \verb+Rij(7,:)+$=R_{\t\t}$ \\
!latex \verb+Rij(8,:)+$=R_{\t\z}$ \\
!latex \verb+Rij(9,:)+$=R_{\z\z}$ \\
  
  ofmn(1:mn)=zero ; cfmn(1:mn)=zero
  call invfft( mn , im , in ,Rmn(2,:) , ofmn(1:mn) , cfmn(1:mn) , Zmn(2,:) , Nt , Nz , Rij(4,:) , Zij(4,:) , isr , trigm , trign , trigwk )
  
  efmn(1:mn)=zero ; sfmn(1:mn)=zero
  call invfft( mn , im , in , efmn(1:mn) , -im*Rmn(1,:) , +im*Zmn(1,:) , sfmn(1:mn) , Nt , Nz , Rij(5,:) , Zij(5,:) , isr , trigm , trign , trigwk )
  
  efmn(1:mn)=zero ; sfmn(1:mn)=zero
  call invfft( mn , im , in , efmn(1:mn) , +in*Rmn(1,:) , -in*Zmn(1,:) , sfmn(1:mn) , Nt , Nz , Rij(6,:) , Zij(6,:) , isr , trigm , trign , trigwk )
  
  ofmn(1:mn)=zero ; cfmn(1:mn)=zero
  call invfft( mn , im , in , -im*im*Rmn(0,:) , ofmn(1:mn) , cfmn(1:mn) , -im*im*Zmn(0,:) , Nt , Nz , Rij(7,:) , Zij(7,:) , isr , trigm , trign , trigwk )
  
  ofmn(1:mn)=zero ; cfmn(1:mn)=zero
  call invfft( mn , im , in , im*in*Rmn(0,:) , ofmn(1:mn) , cfmn(1:mn) , im*in*Zmn(0,:) , Nt , Nz , Rij(8,:) , Zij(8,:) , isr , trigm , trign , trigwk )
  
  ofmn(1:mn)=zero ; cfmn(1:mn)=zero
  call invfft( mn , im , in ,-in*in*Rmn(0,:) , ofmn(1:mn) , cfmn(1:mn) , -in*in*Zmn(0,:) , Nt , Nz , Rij(9,:) , Zij(9,:) , isr , trigm , trign , trigwk )
  
  sg(1,:) = Rij(1,:) * ( Rij(1,:)*Zij(2,:) - Rij(2,:)*Zij(1,:) ) + Rij(0,:) * ( Rij(4,:)*Zij(2,:) - Rij(5,:)*Zij(1,:) + Rij(1,:)*Zij(5,:) - Rij(2,:)*Zij(4,:) )
  sg(2,:) = Rij(2,:) * ( Rij(1,:)*Zij(2,:) - Rij(2,:)*Zij(1,:) ) + Rij(0,:) * ( Rij(5,:)*Zij(2,:) - Rij(7,:)*Zij(1,:) + Rij(1,:)*Zij(7,:) - Rij(2,:)*Zij(5,:) )  
  sg(3,:) = Rij(3,:) * ( Rij(1,:)*Zij(2,:) - Rij(2,:)*Zij(1,:) ) + Rij(0,:) * ( Rij(6,:)*Zij(2,:) - Rij(8,:)*Zij(1,:) + Rij(1,:)*Zij(8,:) - Rij(2,:)*Zij(6,:) )  
  
  do ii=1,3
   do ij=ii,3 ; call gi00aa(ii,ij,ig)
    
    do ik=1,3 ; call gi00aa(ii,ik,ie) ; call gi00aa(ij,ik,if)
     
     gij(ig,ik,:) = Rij(3+ie,:)*Rij(ij,:) + Rij(ii,:)*Rij(3+if,:) + Zij(3+ie,:)*Zij(ij,:) + Zij(ii,:)*Zij(3+if,:)
     
     if( ii.eq.3 .and. ij.eq.3 ) gij(ig,ik,:) = gij(ig,ik,:) + two * Rij(ik,:) * Rij(0,:)
     
    enddo
    
   enddo
  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(co01aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine co01aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
