!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Calculates volume integrals using Gaussian quadrature over radial coordinate.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine ma00aa(Nquad)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, one
  use numerical, only : vsmall
  use fileunits, only : ounit
  use inputlist, only : Wma00aa, Ni, Nofe
  use cputiming, only : Tma00aa
  use allglobal, only : myid, numprocs, cpus, &                          ! 
                        mn, im, in,  ll, &                               ! Fourier resolution, volume index;
                        agss, agst, agsz, agtt, agtz, agzz, agoo, &      ! averaged metric quantities;
                        igss, igst, igsz, igtt, igtz, igzz, igoo, &      ! geometric information;
                        sgss, sgst, sgsz, sgtt, sgtz, sgzz, sgoo, &      ! geometric information; innermost volume;
                        igaa, igab, igba, igbb, &                        ! geometric information required near coordinate singularity;
                       !Ajip, Bjip, &                                    ! coordinate origin regularity matching conditions;
                        felr, ss, dss, Lrho, rho                         ! finite element basis functions, radial subgrid;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS

  INTEGER, intent(in) :: Nquad ! convenient to have this global variable as an argument to eliminate allocation statements;

  INTEGER :: ii, il, ip, ir, iq, ij, ik, jj, ipq, ipqmo, ipqmt, id01bcf, jk, kj, itype, io, oi
  INTEGER :: mjtp, mktq, imj, imk, lmj, lmk

  REAL    :: gwt(1:Nquad), abscis(1:Nquad) ! Gaussian integration weights and abscissae;

  REAL    :: lss, aa, bb, cc, dd, gf(1:7)
  REAL    :: gssjk, gstjk, gszjk, gttjk, gtzjk, gzzjk, goojk
  REAL    :: gsskj, gstkj, gszkj, gttkj, gtzkj, gzzkj, gookj
  REAL    :: rmjtp(-1:2), rmktq(-1:2)
  REAL    :: smj(-1:0), smk(-1:0)
  REAL    :: sgab(5:6), sgba(5:6)

  BEGIN(ma00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wma00aa ) then ; cput= MPI_WTIME() ; write(ounit,'("ma00aa : "f10.2" : ll="i3" ;")')cput-cpus,ll
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \heading{geometric information required in annular volumes}
!latex \item The geometric information is defined
!latex \be \mbox{\type{igss(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}  \varphi_{r,q}  \braket{s_j}{\bar g_{ss}}{s_k} ,\\
!latex     \mbox{\type{igst(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}  \varphi_{r,q}' \braket{s_j}{\bar g_{st}}{c_k} ,\\
!latex     \mbox{\type{igsz(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}  \varphi_{r,q}' \braket{s_j}{\bar g_{sz}}{c_k} ,\\
!latex     \mbox{\type{igtt(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}' \varphi_{r,q}' \braket{c_j}{\bar g_{tt}}{c_k} ,\\
!latex     \mbox{\type{igtz(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}' \varphi_{r,q}' \braket{c_j}{\bar g_{tz}}{c_k} ,\\
!latex     \mbox{\type{igzz(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}' \varphi_{r,q}' \braket{c_j}{\bar g_{zz}}{c_k} ,\\
!latex     \mbox{\type{igoo(l,p,r,q,jk)\%s(i)}} & \equiv & \intdsl \varphi_{l,p}  \varphi_{r,q}' \braket{c_j}{     1     }{c_k} ,\\
!latex \ee
!latex where $l=0,1$, $p=0,$\verb+Nofe+, $r=0,1$, $q=0,$\verb+Nofe+, $j,k=1,$\verb+mn+.
!latex Note that the \verb+j,k+ dependence has been compressed into a single index, \verb+jk=1,mn**2+.

!latex \item The quantities \verb+agss(jk)+ $\equiv \ds \braket{s_j}{\bar g_{ss}}{s_k}$ etc. are computed in \verb+me00ab+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{geometric information required near coordinate singularity}
!latex \item In addition to the geometric information described above, 
!latex the following terms are required by \verb+ma01ac+ to treat the coordinate singularity:
!latex \be \begin{array}{llllllllc}
!latex \mbox{\type{igaa(1,p,q,j,k)}} = \ds \intdso & n_j &           & s^{m_j+2p+2} & n_k &           & s^{m_k+2q+2} & \braket{s_j}{\bar g_{\s\s}}{s_k} \\
!latex \mbox{\type{igaa(2,p,q,j,k)}} = \ds \intdso & n_j &           & s^{m_j+2p+2} &     &(m_k+2q+2) & s^{m_k+2q+1} & \braket{s_j}{\bar g_{\s\z}}{c_k} \\
!latex \mbox{\type{igaa(3,p,q,j,k)}} = \ds \intdso &     &(m_j+2p+2) & s^{m_j+2p+1} & n_k &           & s^{m_k+2q+2} & \braket{s_j}{\bar g_{\s\z}}{c_k} \\
!latex \mbox{\type{igaa(4,p,q,j,k)}} = \ds \intdso &     &(m_j+2p+2) & s^{m_j+2p+1} &     &(m_k+2q+2) & s^{m_k+2q+1} & \braket{c_j}{\bar g_{\z\z}}{c_k} \\
!latex \\
!latex \mbox{\type{igab(1,p,q,j,k)}} = \ds \intdso & n_j &           & s^{m_j+2p+2} & m_k &           & s^{m_k+2q+0} & \braket{s_j}{\bar g_{\s\s}}{s_k} \\
!latex \mbox{\type{igab(2,p,q,j,k)}} = \ds \intdso & n_j &           & s^{m_j+2p+2} &     &(m_k+2q  ) & s^{m_k+2q-1} & \braket{s_j}{\bar g_{\s\t}}{c_k} \\
!latex \mbox{\type{igab(3,p,q,j,k)}} = \ds \intdso &     &(m_j+2p+2) & s^{m_j+2p+1} & m_k &           & s^{m_k+2q+0} & \braket{s_j}{\bar g_{\s\z}}{c_k} \\
!latex \mbox{\type{igab(4,p,q,j,k)}} = \ds \intdso &     &(m_j+2p+2) & s^{m_j+2p+1} &     &(m_k+2q  ) & s^{m_k+2q-1} & \braket{c_j}{\bar g_{\t\z}}{c_k} \\
!latex \mbox{\type{igab(5,p,q,j,k)}} = \ds \intdso &     &           & s^{m_j+2p+2} &     &(m_k+2q  ) & s^{m_k+2q-1} & \braket{c_j}{     1       }{c_k} \\
!latex \mbox{\type{igab(6,p,q,j,k)}} = \ds \intdso &     &(m_j+2p+2) & s^{m_j+2p+1} &     &           & s^{m_k+2q  } & \braket{c_j}{     1       }{c_k} \\
!latex \\
!latex \mbox{\type{igba(1,p,q,j,k)}} = \ds \intdso & m_j &           & s^{m_j+2p+0} & n_k &           & s^{m_k+2q+2} & \braket{s_j}{\bar g_{\s\s}}{s_k} \\
!latex \mbox{\type{igba(2,p,q,j,k)}} = \ds \intdso &     &(m_j+2p  ) & s^{m_j+2p-1} & n_k &           & s^{m_k+2q+2} & \braket{s_j}{\bar g_{\s\t}}{c_k} \\
!latex \mbox{\type{igba(3,p,q,j,k)}} = \ds \intdso & m_j &           & s^{m_j+2p+0} &     &(m_k+2q+2) & s^{m_k+2q+1} & \braket{s_j}{\bar g_{\s\z}}{c_k} \\
!latex \mbox{\type{igba(4,p,q,j,k)}} = \ds \intdso &     &(m_j+2p  ) & s^{m_j+2p-1} &     &(m_k+2q+2) & s^{m_k+2q+1} & \braket{c_j}{\bar g_{\t\z}}{c_k} \\
!latex \mbox{\type{igba(5,p,q,j,k)}} = \ds \intdso &     &(m_j+2p  ) & s^{m_j+2p-1} &     &           & s^{m_k+2q+2} & \braket{c_j}{     1       }{c_k} \\
!latex \mbox{\type{igba(6,p,q,j,k)}} = \ds \intdso &     &           & s^{m_j+2p+0} &     &(m_k+2q+2) & s^{m_k+2q+1} & \braket{c_j}{     1       }{c_k} \\
!latex \\
!latex \mbox{\type{igbb(1,p,q,j,k)}} = \ds \intdso & m_j &           & s^{m_j+2p+0} & m_k &           & s^{m_k+2q  } & \braket{s_j}{\bar g_{\s\s}}{s_k} \\
!latex \mbox{\type{igbb(2,p,q,j,k)}} = \ds \intdso & m_j &           & s^{m_j+2p+0} &     &(m_k+2q  ) & s^{m_k+2q-1} & \braket{s_j}{\bar g_{\s\t}}{c_k} \\
!latex \mbox{\type{igbb(3,p,q,j,k)}} = \ds \intdso &     &(m_j+2p  ) & s^{m_j+2p-1} & m_k &           & s^{m_k+2q  } & \braket{s_j}{\bar g_{\s\t}}{c_k} \\
!latex \mbox{\type{igbb(4,p,q,j,k)}} = \ds \intdso &     &(m_j+2p  ) & s^{m_j+2p-1} &     &(m_k+2q  ) & s^{m_k+2q-1} & \braket{c_j}{\bar g_{\t\t}}{c_k}
!latex \end{array} \ee
!latex \item The above terms are required only in the innermost volume, \verb+ll=1+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{logical control}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do ii = 1 , Ni(ll) ! loop over radial sub-grid;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex \item Within each sub-grid region, the NAG routine \verb+D01CBF+ is called to determine the Gaussian integration abscissae.
!latex The integrals are approximated $\int_a^b f dx \approx \sum_i w_i f(x_i)$.
   
   itype = 0 ; aa = ss(ll)%s(ii-1) ; bb = ss(ll)%s(ii) ; cc = zero ; dd = zero
   
   if( Lrho .and. ll.eq.1 .and. ii.eq.1 ) then ; aa = zero ; bb = one ! normalized integration domain;
   endif

   id01bcf = 0
   CALL(D01BCF , ( itype , aa , bb , cc , dd , Nquad , gwt(1:Nquad) , abscis(1:Nquad) , id01bcf) ) ! sets gaussian weights & abscis;
   
   FATALMESS(ma00aa,id01bcf.ne.0,failed to construct Gaussian integration abscisae and weights)
   
   if( Wma00aa ) then
   !cput= MPI_WTIME() ; write(ounit,'("ma00aa : "f10.2" : myid="i3" ; ll="i3" ; ii="i3" ; abs="99f10.6)')cput-cpus,myid,ll,ii,(abscis(1:Nquad)-aa)/(bb-aa)
    cput= MPI_WTIME() ; write(ounit,'("ma00aa : "f10.2" : myid="i3" ; ll="i3" ; ii="i3" ; abs="99f10.6)')cput-cpus,myid,ll,ii,abscis(1:Nquad)
   endif
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex \item The routine \verb+me00ab+ is called on the sub-sub-grid (i.e. the Gaussian abiscissae) for the evaluation of \verb+agss(j,k)+, etc.
   
   do jj = 1 , Nquad ! loop over radial sub-sub-grid (numerical quadrature);
    
    lss = abscis(jj) ! radial sub-sub-grid determined by Gassian integration;

    if( Lrho .and. ll.eq.1 .and. ii.eq.1 ) lss = lss * rho
    
    ADDCPU(ma00aa) ; CALL(me00ab,(lss)) ; RESETCPU ! calculate averaged `braket' quantities on radial sub-sub-grid; returns agss(1:mn,1:mn), etc.
    
    lss = abscis(jj) ! radial sub-sub-grid determined by Gassian integration;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
    do ip = 0 , Nofe
     do iq = 0 , Nofe ; ipq = ip+iq ; ipqmo = ipq-1 ; ipqmt = ipq-2
      
      do il = 0 , 1 ! note that igaa, igab, igba and igbb do not depend on il, ir;
       do ir = 0 , 1
        
        gf(1) = gwt(jj) * felr(il,ip,0,jj) * dss(ll,ipq  ) * felr(ir,iq,0,jj) ! note that neither    arguments are the differentiated basis function;
        gf(2) = gwt(jj) * felr(il,ip,0,jj) * dss(ll,ipqmo) * felr(ir,iq,1,jj) ! note that the second argument  is  the differentiated basis function;
        gf(3) = gwt(jj) * felr(il,ip,0,jj) * dss(ll,ipqmo) * felr(ir,iq,1,jj) ! note that the second argument  is  the differentiated basis function;
        gf(4) = gwt(jj) * felr(il,ip,1,jj) * dss(ll,ipqmt) * felr(ir,iq,1,jj) ! note that both       arguments are the differentiated basis function;
        gf(5) = gwt(jj) * felr(il,ip,1,jj) * dss(ll,ipqmt) * felr(ir,iq,1,jj) ! note that both       arguments are the differentiated basis function;
        gf(6) = gwt(jj) * felr(il,ip,1,jj) * dss(ll,ipqmt) * felr(ir,iq,1,jj) ! note that both       arguments are the differentiated basis function;
        gf(7) = gwt(jj) * felr(il,ip,0,jj) * dss(ll,ipqmo) * felr(ir,iq,1,jj) ! note that the second argument  is  the differentiated basis function;
        
        do ij = 1 , mn
         do ik = 1 , mn ; jk = ik + (ij-1) * mn ; kj = ij + (ik-1) * mn

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
          
          if( ll.eq.1 .and. ii.eq.1 .and. il.eq.0 .and. ir.eq.0 ) then
           
           gssjk = agss(jk) * gwt(jj)
           gstjk = agst(jk) * gwt(jj)
           gszjk = agsz(jk) * gwt(jj)
           gttjk = agtt(jk) * gwt(jj)
           gtzjk = agtz(jk) * gwt(jj)
           gzzjk = agzz(jk) * gwt(jj)
           goojk = agoo(jk) * gwt(jj)
           
           gsskj = agss(kj) * gwt(jj)
           gstkj = agst(kj) * gwt(jj)
           gszkj = agsz(kj) * gwt(jj)
           gttkj = agtt(kj) * gwt(jj)
           gtzkj = agtz(kj) * gwt(jj)
           gzzkj = agzz(kj) * gwt(jj)
           gookj = agoo(kj) * gwt(jj)

          endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
          
          if( jj.eq.1 ) then ! initialize summation quantities;

           igss(il,ip,ir,iq,jk)%s(ii) = zero
           igst(il,ip,ir,iq,jk)%s(ii) = zero
           igsz(il,ip,ir,iq,jk)%s(ii) = zero
           igtt(il,ip,ir,iq,jk)%s(ii) = zero
           igtz(il,ip,ir,iq,jk)%s(ii) = zero
           igzz(il,ip,ir,iq,jk)%s(ii) = zero
           igoo(il,ip,ir,iq,jk)%s(ii) = zero

           if( ll.eq.1 ) then 
            do imj = 0 , 1
             do imk = 0 , 1
              sgss(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = zero
              sgst(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = zero
              sgsz(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = zero
              sgtt(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = zero
              sgtz(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = zero
              sgzz(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = zero
              sgoo(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = zero
             enddo
            enddo
           endif ! end of if( ll.eq.1 );

          endif ! end of if( jj.eq.1 );

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

          if( ll.eq.1 .and. ii.eq.1 .and. il.eq.0 .and. ir.eq.0 .and. jj.eq.1 ) then ! innermost volume, innermost radial sub-grid; initialize summation quantities;

           igaa(1:4,ip,iq,jk) = zero
           igab(1:6,ip,iq,jk) = zero
           igba(1:6,ip,iq,jk) = zero
           igbb(1:4,ip,iq,jk) = zero

          endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

          igss(il,ip,ir,iq,jk)%s(ii) = igss(il,ip,ir,iq,jk)%s(ii) + gf(1) * agss(jk)
          igst(il,ip,ir,iq,jk)%s(ii) = igst(il,ip,ir,iq,jk)%s(ii) + gf(2) * agst(jk)
          igsz(il,ip,ir,iq,jk)%s(ii) = igsz(il,ip,ir,iq,jk)%s(ii) + gf(3) * agsz(jk)
          igtt(il,ip,ir,iq,jk)%s(ii) = igtt(il,ip,ir,iq,jk)%s(ii) + gf(4) * agtt(jk)
          igtz(il,ip,ir,iq,jk)%s(ii) = igtz(il,ip,ir,iq,jk)%s(ii) + gf(5) * agtz(jk)
          igzz(il,ip,ir,iq,jk)%s(ii) = igzz(il,ip,ir,iq,jk)%s(ii) + gf(6) * agzz(jk)
          igoo(il,ip,ir,iq,jk)%s(ii) = igoo(il,ip,ir,iq,jk)%s(ii) + gf(7) * agoo(jk)

          if( ll.eq.1 ) then
           
           do imj = 0 , 1 ; lmj = im(ij)+2*imj ; smj(-1:0) = (/ lss**(lmj-1) , lss**(lmj) /)
            do imk = 0 , 1 ; lmk = im(ik)+2*imk ; smk(-1:0) = (/ lss**(lmk-1) , lss**(lmk) /)
             
             if( lmj.ge.0 .and. lmk.ge.0 ) sgss(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = igss(il,ip,ir,iq,jk)%s(ii) + gf(1) * agss(jk) * smj( 0) * smk( 0)            
             if( lmj.ge.0 .and. lmk.gt.0 ) sgst(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = igst(il,ip,ir,iq,jk)%s(ii) + gf(2) * agst(jk) * smj( 0) * smk(-1)       * lmk
             if( lmj.ge.0 .and. lmk.gt.0 ) sgsz(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = igsz(il,ip,ir,iq,jk)%s(ii) + gf(3) * agsz(jk) * smj( 0) * smk(-1)       * lmk
             if( lmj.gt.0 .and. lmk.gt.0 ) sgtt(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = igtt(il,ip,ir,iq,jk)%s(ii) + gf(4) * agtt(jk) * smj(-1) * smk(-1) * lmj * lmk
             if( lmj.gt.0 .and. lmk.gt.0 ) sgtz(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = igtz(il,ip,ir,iq,jk)%s(ii) + gf(5) * agtz(jk) * smj(-1) * smk(-1) * lmj * lmk
             if( lmj.gt.0 .and. lmk.gt.0 ) sgzz(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = igzz(il,ip,ir,iq,jk)%s(ii) + gf(6) * agzz(jk) * smj(-1) * smk(-1) * lmj * lmk
             if( lmj.ge.0 .and. lmk.gt.0 ) sgoo(1+imj+2*imk,il,ip,ir,iq,jk)%s(ii) = igoo(il,ip,ir,iq,jk)%s(ii) + gf(7) * agoo(jk) * smj( 0) * smk(-1)       * lmk
             
            enddo
           enddo
           
          endif ! end of if( ll.eq.1 );
           
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
          
          if( ll.eq.1 .and. ii.eq.1 .and. il.eq.0 .and. ir.eq.0 ) then ! innermost volume, innermost radial sub-grid ;
           
           mjtp = im(ij) + 2*ip ; rmjtp(-1:2) = lss**(/ (mjtp-1) , (mjtp+0) , (mjtp+1) , (mjtp+2) /)
           mktq = im(ik) + 2*iq ; rmktq(-1:2) = lss**(/ (mktq-1) , (mktq+0) , (mktq+1) , (mktq+2) /)
           
                                           igaa(1,ip,iq,jk) = igaa(1,ip,iq,jk) + in(ij)            * rmjtp( 2) * in(ik)            * rmktq( 2) * gssjk
                                           igaa(2,ip,iq,jk) = igaa(2,ip,iq,jk) + in(ij)            * rmjtp( 2)          * (mktq+2) * rmktq( 1) * gszjk
                                           igaa(3,ip,iq,jk) = igaa(3,ip,iq,jk) +          (mjtp+2) * rmjtp( 1) * in(ik)            * rmktq( 2) * gszkj
                                           igaa(4,ip,iq,jk) = igaa(4,ip,iq,jk) +          (mjtp+2) * rmjtp( 1)          * (mktq+2) * rmktq( 1) * gzzjk

                                           igab(1,ip,iq,jk) = igab(1,ip,iq,jk) + in(ij)            * rmjtp( 2) * im(ik)            * rmktq( 0) * gssjk
           if(                 mktq.ne.0 ) igab(2,ip,iq,jk) = igab(2,ip,iq,jk) + in(ij)            * rmjtp( 2)          * (mktq  ) * rmktq(-1) * gstjk
                                           igab(3,ip,iq,jk) = igab(3,ip,iq,jk) +          (mjtp+2) * rmjtp( 1) * im(ik)            * rmktq( 0) * gszkj
           if(                 mktq.ne.0 ) igab(4,ip,iq,jk) = igab(4,ip,iq,jk) +          (mjtp+2) * rmjtp( 1)          * (mktq  ) * rmktq(-1) * gtzjk
           if(                 mktq.ne.0 ) igab(5,ip,iq,jk) = igab(5,ip,iq,jk) +                     rmjtp( 2)          * (mktq  ) * rmktq(-1) * goojk
                                           igab(6,ip,iq,jk) = igab(6,ip,iq,jk) +          (mjtp+2) * rmjtp( 1)                     * rmktq( 0) * goojk

                                           igba(1,ip,iq,jk) = igba(1,ip,iq,jk) + im(ij)            * rmjtp( 0) * in(ik)            * rmktq( 2) * gssjk
           if( mjtp.ne.0                 ) igba(2,ip,iq,jk) = igba(2,ip,iq,jk) +          (mjtp  ) * rmjtp(-1) * in(ik)            * rmktq( 2) * gstkj
                                           igba(3,ip,iq,jk) = igba(3,ip,iq,jk) + im(ij)            * rmjtp( 0)          * (mktq+2) * rmktq( 1) * gszjk
           if( mjtp.ne.0                 ) igba(4,ip,iq,jk) = igba(4,ip,iq,jk) +          (mjtp  ) * rmjtp(-1)          * (mktq+2) * rmktq( 1) * gtzjk
           if( mjtp.ne.0                 ) igba(5,ip,iq,jk) = igba(5,ip,iq,jk) +          (mjtp  ) * rmjtp(-1)                     * rmktq( 2) * goojk
                                           igba(6,ip,iq,jk) = igba(6,ip,iq,jk) +                     rmjtp( 0)          * (mktq+2) * rmktq( 1) * goojk

                                           igbb(1,ip,iq,jk) = igbb(1,ip,iq,jk) + im(ij)            * rmjtp( 0) * im(ik)            * rmktq( 0) * gssjk
           if(                 mktq.ne.0 ) igbb(2,ip,iq,jk) = igbb(2,ip,iq,jk) + im(ij)            * rmjtp( 0)          * (mktq  ) * rmktq(-1) * gstjk
           if( mjtp.ne.0                 ) igbb(3,ip,iq,jk) = igbb(3,ip,iq,jk) +          (mjtp  ) * rmjtp(-1) * im(ik)            * rmktq( 0) * gstkj
           if( mjtp.ne.0 .and. mktq.ne.0 ) igbb(4,ip,iq,jk) = igbb(4,ip,iq,jk) +          (mjtp  ) * rmjtp(-1)          * (mktq  ) * rmktq(-1) * gttjk
           
          endif ! end of if( ll.eq.1 .and. ii.eq.1 .and. il.eq.0 .and. ir.eq.0 )
          
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!          if( ll.eq.1 .and. ii.eq.1 .and. il.eq.0 .and. ir.eq.0 ) then ! innermost volume, innermost radial sub-grid ;
!           
!          !if( Wma00aa ) then ; cput= MPI_WTIME() ; write(ounit,'("ma00aa : "f10.2" : myid="i3" ; ll="i3" ; constructing igaa, etc. ; lss="es10.2" ;")')cput-cpus,myid,ll,lss
!          !endif
!           
!          do io = 0 , Nofe ; mjti = im(ij) + 2*io ; rmjti(-1:2) = (/ lss**(mjti-1) , lss**(mjti+0) , lss**(mjti+1) , lss**(mjti+2) /)
!          do oi = 0 , Nofe ; mkto = im(ik) + 2*oi ; rmkto(-1:2) = (/ lss**(mkto-1) , lss**(mkto+0) , lss**(mkto+1) , lss**(mkto+2) /)
!             
!                                          igaa(1,ip,iq,jk) = igaa(1,ip,iq,jk) + in(ij)*in(ik)         *rmjti( 2)*Ajip(ij,io,ip)         *rmkto( 2)*Ajip(ik,oi,iq)*gssjk
!                                          igaa(2,ip,iq,jk) = igaa(2,ip,iq,jk) + in(ij)                *rmjti( 2)*Ajip(ij,io,ip)*(mkto+2)*rmkto( 1)*Ajip(ik,oi,iq)*gszjk
!                                          igaa(3,ip,iq,jk) = igaa(3,ip,iq,jk) +        in(ik)*(mjti+2)*rmjti( 1)*Ajip(ij,io,ip)         *rmkto( 2)*Ajip(ik,oi,iq)*gszkj
!                                          igaa(4,ip,iq,jk) = igaa(4,ip,iq,jk) +               (mjti+2)*rmjti( 1)*Ajip(ij,io,ip)*(mkto+2)*rmkto( 1)*Ajip(ik,oi,iq)*gzzjk
!
!                                          igab(1,ip,iq,jk) = igab(1,ip,iq,jk) + in(ij)*im(ik)         *rmjti( 2)*Ajip(ij,io,ip)         *rmkto( 0)*Bjip(ik,oi,iq)*gssjk
!          if(                 mkto.ne.0 ) igab(2,ip,iq,jk) = igab(2,ip,iq,jk) + in(ij)                *rmjti( 2)*Ajip(ij,io,ip)*(mkto  )*rmkto(-1)*Bjip(ik,oi,iq)*gstjk
!                                          igab(3,ip,iq,jk) = igab(3,ip,iq,jk) +        im(ik)*(mjti+2)*rmjti( 1)*Ajip(ij,io,ip)         *rmkto( 0)*Bjip(ik,oi,iq)*gszkj
!          if(                 mkto.ne.0 ) igab(4,ip,iq,jk) = igab(4,ip,iq,jk) +               (mjti+2)*rmjti( 1)*Ajip(ij,io,ip)*(mkto  )*rmkto(-1)*Bjip(ik,oi,iq)*gtzjk
!          if(                 mkto.ne.0 ) igab(5,ip,iq,jk) = igab(5,ip,iq,jk) +                        rmjti( 2)*Ajip(ij,io,ip)*(mkto  )*rmkto(-1)*Bjip(ik,oi,iq)*goojk
!                                          igab(6,ip,iq,jk) = igab(6,ip,iq,jk) +               (mjti+2)*rmjti( 1)*Ajip(ij,io,ip)         *rmkto( 0)*Bjip(ik,oi,iq)*goojk
!
!                                          igba(1,ip,iq,jk) = igba(1,ip,iq,jk) + im(ij)*in(ik)         *rmjti( 0)*Bjip(ij,io,ip)         *rmkto( 2)*Ajip(ik,oi,iq)*gssjk
!          if( mjti.ne.0                 ) igba(2,ip,iq,jk) = igba(2,ip,iq,jk) +        in(ik)*(mjti  )*rmjti(-1)*Bjip(ij,io,ip)         *rmkto( 2)*Ajip(ik,oi,iq)*gstkj
!                                          igba(3,ip,iq,jk) = igba(3,ip,iq,jk) + im(ij)                *rmjti( 0)*Bjip(ij,io,ip)*(mkto+2)*rmkto( 1)*Ajip(ik,oi,iq)*gszjk
!          if( mjti.ne.0                 ) igba(4,ip,iq,jk) = igba(4,ip,iq,jk) +               (mjti  )*rmjti(-1)*Bjip(ij,io,ip)*(mkto+2)*rmkto( 1)*Ajip(ik,oi,iq)*gtzjk
!          if( mjti.ne.0                 ) igba(5,ip,iq,jk) = igba(5,ip,iq,jk) +               (mjti  )*rmjti(-1)*Bjip(ij,io,ip)         *rmkto( 2)*Ajip(ik,oi,iq)*goojk
!                                          igba(6,ip,iq,jk) = igba(6,ip,iq,jk) +                        rmjti( 0)*Bjip(ij,io,ip)*(mkto+2)*rmkto( 1)*Ajip(ik,oi,iq)*goojk
!
!                                          igbb(1,ip,iq,jk) = igbb(1,ip,iq,jk) + im(ij)*im(ik)         *rmjti( 0)*Bjip(ij,io,ip)         *rmkto( 0)*Bjip(ik,oi,iq)*gssjk
!          if(                 mkto.ne.0 ) igbb(2,ip,iq,jk) = igbb(2,ip,iq,jk) + im(ij)                *rmjti( 0)*Bjip(ij,io,ip)*(mkto  )*rmkto(-1)*Bjip(ik,oi,iq)*gstjk
!          if( mjti.ne.0                 ) igbb(3,ip,iq,jk) = igbb(3,ip,iq,jk) +        im(ik)*(mjti  )*rmjti(-1)*Bjip(ij,io,ip)         *rmkto( 0)*Bjip(ik,oi,iq)*gstkj
!          if( mjti.ne.0 .and. mkto.ne.0 ) igbb(4,ip,iq,jk) = igbb(4,ip,iq,jk) +               (mjti  )*rmjti(-1)*Bjip(ij,io,ip)*(mkto  )*rmkto(-1)*Bjip(ik,oi,iq)*gttjk
!           
!          enddo ! end of do io;
!          enddo ! end of do oi; 
!  
!          !if( Wma00aa ) then ; cput= MPI_WTIME() ; write(ounit,'("ma00aa : "f10.2" : myid="i3" ; ll="i3" ; constructed  igaa, etc. ;")')cput-cpus,myid,ll
!          !endif
!           
!          endif ! end of if( ll.eq.1 .and. ii.eq.1 .and. il.eq.0 .and. ir.eq.0 )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

         enddo ! end of do ik;
        enddo ! end of do ij;
        
       enddo ! end of do iq;
      enddo ! end of do ir;
     enddo ! end of do ip;
    enddo ! end of do il;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   enddo ! end of do jj; loop over radial sub-sub-grid; this is the summation loop;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   if( Lrho .and. ll.eq.1 .and. ii.eq.1 ) then ! need to scale the igaa, etc.
    do ip = 0 , Nofe
     do iq = 0 , Nofe

      igaa(1,ip,iq,1:mn*mn) = igaa(1,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip+0 + 2+2*iq+0 )
      igaa(2,ip,iq,1:mn*mn) = igaa(2,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip+0 + 2+2*iq-1 )
      igaa(3,ip,iq,1:mn*mn) = igaa(3,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip-1 + 2+2*iq+0 )
      igaa(4,ip,iq,1:mn*mn) = igaa(4,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip-1 + 2+2*iq-1 )

      igab(1,ip,iq,1:mn*mn) = igab(1,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip+0 + 0+2*iq+0 )
      igab(2,ip,iq,1:mn*mn) = igab(2,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip+0 + 0+2*iq-1 )
      igab(3,ip,iq,1:mn*mn) = igab(3,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip-1 + 0+2*iq+0 )
      igab(4,ip,iq,1:mn*mn) = igab(4,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip-1 + 0+2*iq-1 )
      igab(5,ip,iq,1:mn*mn) = igab(5,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip+0 + 0+2*iq-1 )
      igab(6,ip,iq,1:mn*mn) = igab(6,ip,iq,1:mn*mn) * rho**( 1 + 2+2*ip-1 + 0+2*iq+0 )

      igba(1,ip,iq,1:mn*mn) = igba(1,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip+0 + 2+2*iq+0 )
      igba(2,ip,iq,1:mn*mn) = igba(2,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip-1 + 2+2*iq+0 )
      igba(3,ip,iq,1:mn*mn) = igba(3,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip+0 + 2+2*iq-1 )
      igba(4,ip,iq,1:mn*mn) = igba(4,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip-1 + 2+2*iq-1 )
      igba(5,ip,iq,1:mn*mn) = igba(5,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip-1 + 2+2*iq+0 )
      igba(6,ip,iq,1:mn*mn) = igba(6,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip+0 + 2+2*iq-1 )

      igbb(1,ip,iq,1:mn*mn) = igbb(1,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip+0 + 0+2*iq+0 )
      igbb(2,ip,iq,1:mn*mn) = igbb(2,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip+0 + 0+2*iq-1 )
      igbb(3,ip,iq,1:mn*mn) = igbb(3,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip-1 + 0+2*iq+0 )
      igbb(4,ip,iq,1:mn*mn) = igbb(4,ip,iq,1:mn*mn) * rho**( 1 + 0+2*ip-1 + 0+2*iq-1 )

     enddo ! end of do ip;
    enddo ! end of do iq;
   endif ! end of if( ll.eq.1 .and. ii.eq.1 );
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of do ii; loop over radial sub-grid;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!  if( Wma00aa .and. ll.eq.1 ) then
!
!   cput= MPI_WTIME()
!
!   lss = ss(1)%s(1) 
!
!   do ij = 1 , mn
!    do ik = 1 , mn ; jk = ik + (ij-1) * mn ; kj = ij + (ik-1) * mn
!
!     do ip = 0 , Nofe
!      do iq = 0 , Nofe
!
!       sgab(5:6) = zero ; sgba(5:6) = zero ! initialize summation;
!       
!       do io = 0 , Nofe ; mjti = im(ij) + 2*io 
!        do oi = 0 , Nofe ; mkto = im(ik) + 2*oi 
!
!         sgab(5) = sgab(5) + agoo(jk) * Ajip(ij,io,ip) * Bjip(ik,oi,iq) * (mkto  ) * lss**( mjti+2+mkto-1+1 ) / ( mjti+2+mkto-1+1 )
!         sgab(6) = sgab(6) + agoo(jk) * Ajip(ij,io,ip) * Bjip(ik,oi,iq) * (mjti+2) * lss**( mjti+1+mkto  +1 ) / ( mjti+1+mkto  +1 )
!
!         sgba(5) = sgba(5) + agoo(jk) * Bjip(ij,io,ip) * Ajip(ik,oi,iq) * (mjti  ) * lss**( mjti-1+mkto+2+1 ) / ( mjti-1+mkto+2+1 )
!         sgba(6) = sgba(6) + agoo(jk) * Bjip(ij,io,ip) * Ajip(ik,oi,iq) * (mkto+2) * lss**( mjti  +mkto+1+1 ) / ( mjti  +mkto+1+1 )
!
!        enddo ! end of do oi;
!       enddo ! end of do io;
!
!       write(ounit,1000)im(ij),in(ij),im(ik),in(ik),ip,iq,sgba(6),igba(6,ip,iq,jk),abs(sgba(6)-igba(6,ip,iq,jk))
!
!      enddo ! end of do iq;
!     enddo ! end of do ip;
!
!     pause
!
!    enddo ! end of do ik;
!   enddo ! end of do ij;
!   
!1000 format("ma00aa : sgba(6) : (m,n),(m,n)="4i3" (p,q)="2i2" ; "2es23.15" ; err="es10.2" ;")
!
!  endif ! end of if( Wma00aa .and. ll.eq.1 )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!  if( Wma00aa .and. ll.eq.1 ) then
!   
!   cput= MPI_WTIME()
!   write(ounit,'("ma00aa : "f10.2" : ll="i3" ;")')cput-cpus,ll
!   
!   do ip = 0 , Nofe
!    do iq = 0 , Nofe
!     do ij = 1 , mn
!      do ik = 1 , mn  ; jk = ik + (ij-1) * mn ; kj = ij + (ik-1) * mn
!       write(ounit,*)
!       write(ounit,'("ma00aa : igab(1,"i1","i1","i2","i2") = "2es23.15" = igba(1,"i1","i1","i2","i2") ; "es9.2)')&
!     iq,ip,ik,ij,igab(1,iq,ip,kj),igba(1,ip,iq,jk),ip,iq,ij,ik,abs(igab(1,iq,ip,kj)-igba(1,ip,iq,jk))
!       write(ounit,'("ma00aa : igab(2,"i1","i1","i2","i2") = "2es23.15" = igba(2,"i1","i1","i2","i2") ; "es9.2)')&
!     iq,ip,ik,ij,igab(2,iq,ip,kj),igba(2,ip,iq,jk),ip,iq,ij,ik,abs(igab(2,iq,ip,kj)-igba(2,ip,iq,jk))
!       write(ounit,'("ma00aa : igab(3,"i1","i1","i2","i2") = "2es23.15" = igba(3,"i1","i1","i2","i2") ; "es9.2)')&
!     iq,ip,ik,ij,igab(3,iq,ip,kj),igba(3,ip,iq,jk),ip,iq,ij,ik,abs(igab(3,iq,ip,kj)-igba(3,ip,iq,jk))
!       write(ounit,'("ma00aa : igab(4,"i1","i1","i2","i2") = "2es23.15" = igba(4,"i1","i1","i2","i2") ; "es9.2)')&
!     iq,ip,ik,ij,igab(4,iq,ip,kj),igba(4,ip,iq,jk),ip,iq,ij,ik,abs(igab(4,iq,ip,kj)-igba(4,ip,iq,jk))
!       write(ounit,'("ma00aa : igab(5,"i1","i1","i2","i2") = "2es23.15" = igba(5,"i1","i1","i2","i2") ; "es9.2)')&
!     iq,ip,ik,ij,igab(5,iq,ip,kj),igba(5,ip,iq,jk),ip,iq,ij,ik,abs(igab(5,iq,ip,kj)-igba(5,ip,iq,jk))
!       write(ounit,'("ma00aa : igab(6,"i1","i1","i2","i2") = "2es23.15" = igba(6,"i1","i1","i2","i2") ; "es9.2)')&
!     iq,ip,ik,ij,igab(6,iq,ip,kj),igba(6,ip,iq,jk),ip,iq,ij,ik,abs(igab(6,iq,ip,kj)-igba(6,ip,iq,jk))
!
!       if( abs(igab(1,iq,ip,kj)-igba(1,ip,iq,jk)) .gt.vsmall .or. &
!          !abs(igab(2,iq,ip,kj)-igba(2,ip,iq,jk)) .gt.vsmall .or. &
!          !abs(igab(3,iq,ip,kj)-igba(3,ip,iq,jk)) .gt.vsmall .or. &
!           abs(igab(4,iq,ip,kj)-igba(4,ip,iq,jk)) .gt.vsmall .or. &
!           abs(igab(5,iq,ip,kj)-igba(5,ip,iq,jk)) .gt.vsmall .or. &
!           abs(igab(6,iq,ip,kj)-igba(6,ip,iq,jk)) .gt.vsmall ) pause
!
!      enddo ! end of do ik;
!     enddo ! end of do ij;
!    enddo ! end of do iq;
!   enddo ! end of do ip;
!
!  endif ! end of if( Wma00aa );
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(ma00aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine ma00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
