!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Adjusts helicity-multipler, $\mu$, and poloidal flux, $\delta \psi_p$, to solve transform constraint.
!latex This routine is only called by \verb+fc00aa+, \verb+fc01aa+.

!latex \heading{rotational transform constraint}
!latex \item In each annulus, considering the enclosed toroidal flux, $\delta \psi_t$, to be given, there are two degrees of freedom in the Beltrami fields
!latex that are not constrained by the gauge or the condition that the field be tangential to the adjacent interfaces.
!latex These two degrees of freedom are the helicity multiplier, $\mu$, and the enclosed poloidal flux, $\delta \psi_p$.
!latex \item Also, in each annulus there are two constraints, namely that the rotational transform match prescribed values.
!latex \item This routine adjusts ${\bf x}\equiv(\mu,\delta \psi_p)$ so that ${\bf f}({\bf x})\equiv (\iotabar_{inn}-\iotabar_{l-1}, \iotabar_{out}-\iotabar_{l})=0$,
!latex where $\iotabar_{inn}$and $\iotabar_{out}$ are the `measured' rotational-transform on the inner and outer interfaces,
!latex and $\iotabar_l$ are the required rotational-transforms prescribed on input.
!latex The `measured' rotational-transform on the interfaces is determined by constructing the Beltrami field 
!latex (for a given $\mu$ and $\delta \psi_p$, see \verb+mp00aa+),
!latex then performing a transformation to a straight-field line angle on the interfaces (see \verb+tr00ab+).

!latex  \item In the innermost volume, the poloidal flux is not a degree of freedom and only $\mu$ is adjusted to satisfy the single transform constraint.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine ma02aa( liota , ierror )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero
  use numerical, only : !vsmall
  use fileunits, only : ounit
  use inputlist, only : Wma02aa, mu, mupftol, tflux, Ldense, Lsparse
  use cputiming
  use allglobal, only : myid, cpus, ll, mn, im, in, NBdof, MB, MR, dpflux, SCSF, irow, icol!, rho, iss, Atmn, Azmn

  use mupfcount, only : imupf ! iteration counter; initialized below; incremented in mp00aa;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  LOGICAL, intent(in)  :: liota
  INTEGER, intent(out) :: ierror
  
  LOGICAL              :: Lassign

  INTEGER              :: iflag, ic05nbf, Nnz, Nmudp, Lmudpwk!, imn, ilss, nilss
  REAL                 :: Xmudp(2), Fmudp(2), tol!, lss
  REAL   , allocatable :: mudpwk(:)

  external :: mp00aa

  BEGIN(ma02aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \heading{allocation}
!latex \item This routine allocates the arrays \verb+MB(0:NBdof(ll),0:NBdof(ll))+, only \verb+if(Ldense)+, and \verb+MR(0:NBdof(ll))+,
!latex which are assigned in \verb+ma01aa+, which is called by \verb+mp00aa+.
!latex Note that only \verb+MB(1:NBdof(ll),1:NBdof(ll))+ and \verb+MR(1:NBdof(ll))+ are actually used.

!latex If the sparse representation of the Beltrami linear system is employed, \verb+if(Lsparse)+, then \verb+SCSF+, \verb+irow+ and \verb+icol+ are also allocated.
  
  if( Wma02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("ma02aa : "f10.2" : myid="i3" ; ll="i3" ; allocating MR, MB etc ;")')cput-cpus,myid,ll
  endif

  RALLOCATE(MR,(0:NBdof(ll))) ! driving rhs; combined energy and helicity parts;
  
  if( Ldense ) then
   RALLOCATE(MB,(0:NBdof(ll),0:NBdof(ll))) ! total linear system; MB and MR are assigned in ma01aa (which is called by mp00aa);
  endif
  
  if( Lsparse ) then
   
   ADDCPU(ma02aa)

   Lassign = .false. ; Nnz = 0
   FATALMESS(ma02aa,.true.,ma01aa has been removed : need to implement sparse representation into ma01ab/ma01ac)
  !CALL(ma01aa,(Lassign,Nnz)) ! count #non-zero entries in Beltrami linear system; repeated entries exist ?;
   
   RESETCPU

   RALLOCATE(SCSF,(1:Nnz)) ! total linear system; sparse representation of Beltrami linear matrix;
   RALLOCATE(irow,(1:Nnz)) ! total linear system; sparse representation of Beltrami linear matrix;
   RALLOCATE(icol,(1:Nnz)) ! total linear system; sparse representation of Beltrami linear matrix;
   
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!  if( ll.eq.1 ) then ; Nmudp = 1 ; Xmudp(1:1) = (/ mu(ll)              /) ! there  is only one degree  of freedom in the innermost volume, namely mu           ;
!  else               ; Nmudp = 2 ; Xmudp(1:2) = (/ mu(ll) , dpflux(ll) /) ! there are      two degrees of freedom in the innermost volume, namely mu and dpflux;
!  endif   
  Nmudp = 2 ; Xmudp(1:2) = (/ mu(ll) , dpflux(ll) /) ! two degrees of freedom;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  iflag = 0 ; ierror = 0

  imupf = 0 ! iteration counter; initialized here; incremented in mp00aa;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{logical control}
  
  select case(liota)
   
!latex \item If \verb+Liota=F+, the helicity-multiplier, $\mu$, and poloidal flux, $\delta\psi_p$, are {\em not} adjusted to match interface transforms.
   
  case(.false.) 
   
   if( Wma02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("ma02aa : "f10.2" : myid="i3" ; ll="i3" ; calling mp00aa ;")')cput-cpus,myid,ll
   endif
   
   ADDCPU(ma02aa)

   CALL(mp00aa,(Nmudp,Xmudp(1:Nmudp),Fmudp(1:Nmudp),iflag)) ! mp00aa solves for Beltrami field;
   
   RESETCPU
   
   if( Wma02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("ma02aa : "f10.2" : myid="i3" ; ll="i3" ; called  mp00aa ;")')cput-cpus,myid,ll
   endif

   if( iflag.eq.-3 ) ierror = 1 ! linear solver routine has failed in mp00aa and error flag has been set;
   
!latex \item If \verb+Liota=T+, the NAG routine \verb+c05nbf+ is used to adjust $\mu$ and $\delta\psi_p$ to match the transform constraints.
!latex \item The input \verb+mupftol+ controls the accuracy to which $\mu$ and $\delta\psi_p$ are required.
!latex \item If the iteration count is equal to \verb+mupfits+, then the iterations are terminated (with \verb+iflag=-2+, see \verb+mp00aa+ for details).

  case(.true.) 
   
   if( Wma02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("ma02aa : "f10.2" : myid="i3" ; ll="i3" ; calling mp00aa iteratively through C05NBF ;")')cput-cpus,myid,ll
   endif
   
   tol = mupftol ; Lmudpwk = Nmudp * ( 3 * Nmudp + 13 ) / 2 + 1 ; ic05nbf = 1
   
   RALLOCATE(mudpwk,(Lmudpwk))
   
   ADDCPU(ma02aa)
   
   CALL(C05NBF,(mp00aa,Nmudp,Xmudp(1:Nmudp),Fmudp(1:Nmudp),tol,mudpwk,Lmudpwk,ic05nbf))

   RESETCPU

   DEALLOCATE(mudpwk)
   
!latex \item If \verb+mp000aa+ encounters an error calculating the Beltrami field, then the error flag \verb+iflag=-3+ is returned from \verb+mp00aa+
!latex and \verb+ma02aa+ then returns \verb+ierror=1+.

   if( ic05nbf.eq.-3 ) ierror = 1 ! linear solver routine has failed in mp00aa and error flag has been set;
   
   FATALMESS(ma02aa,ic05nbf.gt.0,could not adjust mu and dpflux in order to satisfy transform constraint)

   if( Wma02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("ma02aa : "f10.2" : myid="i3" ; ll="i3" ; called  mp00aa iteratively through C05NBF ;")')cput-cpus,myid,ll
   endif
   
  end select
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  if( Wma02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("ma02aa : "f10.2" : myid="i3" ; ll="i3" ; de-allocating MR, MB etc ;")')cput-cpus,myid,ll
  endif

  if( Ldense ) then ; DEALLOCATE(MB)
  endif
  
  if( Lsparse ) then ; DEALLOCATE(SCSF) ; DEALLOCATE(irow) ; DEALLOCATE(icol)
  endif
  
  DEALLOCATE(MR)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(ma02aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine ma02aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


