!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Solves Beltrami linear system (for given helicity multiplier and poloidal flux),
!latex and returns interface rotational transform (via call to tr00ab).

!latex \item This routine is only called by \verb+ma02aa+, usually via \verb+C05NBF+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine mp00aa( Nmudp , Xmudp , Fmudp , iflag )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero, half, goldmean
  use numerical, only : machprec, sqrtmachprec, small, vsmall
  use fileunits, only : ounit
  use inputlist, only : Wmp00aa, Wma02aa, Nvol, Ni, Nofe, tflux, mu, pl, pr, ql, qr, Ldense, Lsparse, Lposdef, ssoromega, sparsetol, sparseits, sparsepc, mupfits, mupftol
  use cputiming, only : Tmp00aa
  use allglobal, only : mn, im, in, mns, ll, cpus, myid, NBdof, MB, MR, At, Az, dpflux, sfliota, SCSF, irow, icol, &
                        igss, igst, igsz, igtt, igtz, igzz, igoo, ijk

  use mupfcount, only : imupf ! iteration counter; initialized in ma02aa; incremented below;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  INTEGER, intent(in)  :: Nmudp
  REAL   , intent(in)  :: Xmudp(1:Nmudp)
  REAL   , intent(out) :: Fmudp(1:Nmudp)
  INTEGER              :: iflag

  LOGICAL              :: Lassign
  INTEGER              :: ii, jj, ij, ip, ix, ik, iq, jx, itz, jtz, if04asf, if04atf, if11zbf, if11jef, Nnz, Nnzo, NN, Lrwk, IA, IAA, maxitn, itn, M1, M2, im01caf, iprint=17
  INTEGER, allocatable :: iwk(:), istr(:), swk(:) ! for sorting/summing SCSF array;
  REAL                 :: omega, tol, rnorm, iotas(0:1), iotad(0:1), iotaerr(0:1), xierr, cpul, dcpu, scpu, error, cpua, cpub, cpuc
  REAL   , allocatable :: xi(:), xio(:), wk1(:), wk2(:), rwk(:), AA(:,:)
  CHARACTER            :: meth*6, pc*1, ORDER*1

  INTEGER              :: ljk,il,ir

  BEGIN(mp00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

                   mu(ll)     = Xmudp(1) ! Beltrami system depends on helicity multipler, mu;
  if( Nmudp.eq.2 ) dpflux(ll) = Xmudp(2) ! Beltrami system depends on enclosed poloidal flux; dpflux is initialized in global;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  sfliota(ll,0:1) = zero ! iota will only be calculated if Nofe.gt.0; this is passed through global;
  
  NN = NBdof(ll) ! shorthand;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{logical control}

!latex \item First, the matrix is assigned via a call to \verb+ma01ae+.
!latex This assigns the matrix quantities in \verb+MR+, and \verb+MB+, only \verb+if(Ldense)+, and \verb+SCSF+, \verb+irow+ and \verb+icol+, only \verb+if(Lsparse)+.
!latex (These arrays were allocated in \verb+ma02aa+.)
  
  ADDCPU(mp00aa)

  if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling ma01ae ;")')cput-cpus,myid,ll
  endif

  CALL(ma01ae,(Nofe))
  
  if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; called  ma01ae ;")')cput-cpus,myid,ll
  endif
  
!  if( ll.gt.1 ) then ! compare with old subroutine;
!   
!   RALLOCATE(AA,(1:NN,1:NN)) ! used to compare MB from ma01ae and ma01ab
!   AA(1:NN,1:NN)=MB(1:NN,1:NN)
!   
!   RALLOCATE(rwk,(1:NN)) ! used to compare MR from ma01ae and ma01ab;
!   rwk(1:NN) = MR(1:NN)
!
!   Lassign=.true.
!   
!   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling ma01ab ;")')cput-cpus,myid,ll
!   endif
!   
!   CALL(ma01ab,(Lassign,Nnzo)) ! returns MB and MR, for an arbitrary annulus;
!   
!   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; called  ma01ab ;")')cput-cpus,myid,ll
!   endif
!
!   write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; error MB-MB="es10.2" ; ":"MR-MR="es10.2" ;")')cput-cpus,myid,ll,&
!   sqrt( sum( ( AA(1:NN,1:NN) - MB(1:NN,1:NN) )**2 ) / NN / NN ), sqrt( sum( ( rwk(1:NN) - MR(1:NN) )**2 ) / NN )
!
!   DEALLOCATE(AA)
!   DEALLOCATE(rwk)
!   
!  endif ! end of if( ll.gt.1 );
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RESETCPU

  MR(1:NN) = -MR(1:NN) ! rhs of Beltrami linear system; this is a little strange; should change ma01ae to give -MR directly;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RALLOCATE(xi,(NN)) ! solution vector;
  
!  if( Wmp00aa ) then
!   CALL(up00ab(NN,xi(1:NN))) ! start iterations from last solution;
!   xi = abs(xi)
!   M1=1 ; M2=NN ; ORDER='D' ; im01caf=0 ; call M01CAF(xi(1:NN),M1,M2,ORDER,im01caf) ! this rearranges xi;
!   cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : unpacked xi="999es12.5)')cput-cpus,xi(1:iprint)
!  endif

  dcpu=zero ; scpu=zero

  if( Ldense .and. Lsparse ) then ; RALLOCATE(xio,(NN)) ! xio is used to compare solutions;
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item Different methods may be used to solve the linear system, i.e. to construct the Beltrami field in the given volume.
!latex \begin{itemize}
!latex \item if \verb+Ldense.eq.T+ then
!latex \begin{itemize}
!latex \item if \verb+Lposdef=T+ then : solution is provided by \verb+F04ASF+, which assumes the matrix is symmetric positive-definite;
!latex \item if \verb+Lposdef=F+ then : solution is provided by \verb+F04ATF+;
!latex \end{itemize}
!latex Both these routines require the matrix as an $N \times N$ array.
!latex (Note that \verb+F04ASF+ is faster than \verb+F04ATF+; needs to be checked.)

!latex \item The solution vector is `unpacked' by \verb+up00aa+. The unpacking routine must be consistent with the `packing' description given in \verb+global+.

  if( Ldense ) then
   
   cpul = MPI_WTIME()
   
   IA=NN ; IAA=NN ; if04asf = 0 ; if04atf = 0
   

   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling F04ASF (if "L1") or F04ATF ;")')cput-cpus,myid,ll, Lposdef
   endif
   
   RALLOCATE(wk1,(NN)) ; RALLOCATE(wk2,(NN)) ! maybe these work arrays can be allocated in global;
   
   if( Lposdef ) then ! will assume the matrix is positive definite;
    
    if04asf=1 ; call F04ASF( MB(1:NN,1:NN) , IA , MR(1:NN) , NN , xi(1:NN) , wk1(1:NN) , wk2(1:NN) , if04asf ) ! MB is changed;
    
   else ! will not assume the matrix is positive definite;
    
    RALLOCATE(AA,(IAA,NN))
    
    if04atf=1 ; call F04ATF( MB(1:NN,1:NN) , IA , MR(1:NN) , NN , xi(1:NN) , AA(1:IAA,1:NN) , IAA , wk1(1:NN) , wk2(1:NN) , if04atf )
    
    DEALLOCATE(AA)

   endif ! end of if( Lposdef )
   
   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; if04asf="i2" ; if0atf="i2" ;")')cput-cpus,myid,ll,if04asf,if04atf
   endif


   DEALLOCATE(wk1) ; DEALLOCATE(wk2)
   
   FATALMESS(mp00aa,if04asf.eq.1                  ,Beltrami linear matrix : not positive definite)
   FATALMESS(mp00aa,                  if04atf.eq.1,Beltrami linear matrix : singular)
   FATALMESS(mp00aa,if04asf.eq.2 .or. if04atf.eq.2,Beltrami linear matrix : ill conditioned)
   FATALMESS(mp00aa,if04asf.eq.3 .or. if04atf.eq.3,Beltrami linear matrix : error on entry)
   FATALMESS(mp00aa,if04asf.ne.0 .or. if04atf.ne.0,Beltrami linear matrix : error)
   
   if( if04asf.ne.0 .or. if04atf.ne.0 ) then ! terminate iterations; return error flag to ma02aa;
    iflag = -3
    DEALLOCATE(wk1) ; DEALLOCATE(wk2)
    DEALLOCATE(xi)
    if( Lsparse ) then ; DEALLOCATE(xio)
    endif
    goto 9999
   endif
   
   cput = MPI_WTIME() ; dcpu=cput-cpul ! reqd time for dense linear solver;

   if( Lsparse ) xio(1:NN) = xi(1:NN) ! save solution; will be compared with sparse routine;
   
   ADDCPU(mp00aa)

   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling up00aa ;")')cput-cpus,myid,ll
   endif
   
   CALL(up00aa,(NN,xi(1:NN))) ! unpacking; this assigns At, Az through common; xi is no longer needed;

   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; called  up00aa ;")')cput-cpus,myid,ll
   endif

   if( Lsparse ) then
    CALL(up00ab,(NN,xio(1:NN))) ! repacks
    cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; packing error="es10.2" ;")')cput-cpus,myid,ll,sqrt(sum((xi-xio)**2)/NN)
   endif

   if( Nofe.gt.0 ) then

    if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling tr00ab ;")')cput-cpus,myid,ll
    endif

    CALL(tr00ab,(Nvol,mns,sfliota(ll,0:1))) ; iotad(0:1)=sfliota(ll,0:1) ! At,Az must be valid;

    if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; called  tr00ab ;")')cput-cpus,myid,ll
    endif

    if( Lsparse ) then ; cput = MPI_WTIME() ; write(ounit,1111)cput-cpus,myid,ll,"dense ",sfliota(ll,0:1),dcpu
    endif

   endif
   
   RESETCPU

!   if( Wmp00aa ) then
!    xi = abs(xi)
!    M1=1 ; M2=NN ; ORDER='D' ; im01caf=0 ; call M01CAF(xi(1:NN),M1,M2,ORDER,im01caf) ! this rearranges xi;
!    cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" :  dense   xi="999es12.5)')cput-cpus,xi(1:iprint)
!   endif

  endif ! end of if( Ldense );
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item if \verb+Lsparse.eq.T+ then 
!latex will exploit the sparse structure of the matrix to reduce memory (and increase speed?). The routine \verb+F11JEF+ is used.
!latex Various parameters are given as input:
!latex \begin{itemize}
!latex \item if \verb+Lposdef=T+ : use conjugate gradient, \verb+method='CG'+
!latex \item if \verb+Lposdef=F+ : use Lanczos method, \verb+method='SYMMLQ'+
!latex \item if \verb+sparsepc=0+ : no preconditionder, \verb+precon='N'+ (works terribly);
!latex \item if \verb+sparsepc=1+ : Jacobi preconditionder, \verb+precon='J'+
!latex \item if \verb+sparsepc=2+ : SSOR preconditionder, \verb+precon='S'+
!latex \end{itemize}
!latex Other parameters, \verb+sparsetol+,\verb+sparseits+ and \verb+ssoromega+, are provided on input.
!latex The accuracy of the solution is controlled by \verb+sparsetol+. (See the NAG documentation for details.)
!latex Note that if \verb+sparseits.le.0+, then \verb+sparseits=N+ where \verb+N+ is the size of the matrix.
!latex Note that if \verb+sparsetol.le.0.0+, then \verb+tol=+$\epsilon$ where $\epsilon$ is machine precision.
!latex The initial guess for the iterative calculation is taken to be the last calculation solution.
!latex The sparse iterative approach needs more work to verify the speed, robustness and accuracy.
  
  if( Lsparse ) then
   
   cpul = MPI_WTIME()

!latex \item The sparse representation of the matrix, as determined by \verb+ma01aa+, may contain repeated entries.
!latex These must be summed (by \verb+F11ZBF+).
   
!latex \end{itemize}

! parallel implementation gives different result for multiple volumes : need to check memory,.true.)
   
   ALLOCATE(istr,(NN+1)) ; ALLOCATE(swk,(NN)) ! integer array;
   
! sort sparse matrix storage;
   if11zbf=0 ; Nnz=Nnzo ; call F11ZBF( NN , Nnz , SCSF(1:Nnz) , irow(1:Nnz) , icol(1:Nnz) , 'S' , 'R' , istr(1:NN+1) , swk(1:NN) , if11zbf ) 
   
   FATALMESS(mp00aa,if11zbf.ne.0,error sorting sparse matrix storage)
   
   DEALLOCATE(istr) ; DEALLOCATE(swk)
   
   tol = sparsetol ; omega = ssoromega ; maxitn = sparseits ; itn = 0 ; rnorm = zero
   
   if( maxitn.le.0 ) maxitn = NN
   if( tol.le.zero ) tol = machprec
   
   select case (sparsepc)
   case(0) ; pc='N'
   case(1) ; pc='J'
   case(2) ; pc='S'
   case default
    FATALMESS(mp00aa,.true., invalid value of sparsepc)
   end select
   
   if( Lposdef ) then ; meth='CG'     ; Lrwk=6*NN+120
   else               ; meth='SYMMLQ' ; Lrwk=7*NN+120
   endif
   
   if( pc.eq.'J' .or. pc.eq.'S' ) Lrwk = Lrwk + NN
   
   ALLOCATE(rwk,(Lrwk)) ; ALLOCATE(iwk,(NN+1)) ! for iterative solver;
   
   ADDCPU(mp00aa)

   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling up00ab ;")')cput-cpus,myid,ll
   endif

   xi(1:NN) = zero ; CALL(up00ab(NN,xi(1:NN))) ! start iterations from last solution;
   
   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; called  up00ab ;")')cput-cpus,myid,ll
   endif

   RESETCPU

  !if( Ldense ) then ; xi(1:NN)=zero ! start iterations from arbitrary point to check iterations converge;
  !endif

   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling F11JEF ;")')cput-cpus,myid,ll
   endif
   
   if11jef = 1
   call F11JEF( meth , pc , NN , Nnz , SCSF(1:Nnz) , irow(1:Nnz) , icol(1:Nnz) , omega , MR(1:NN) , tol , maxitn , xi(1:NN) , rnorm , itn , rwk , lrwk , iwk , if11jef )

   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; called  F11JEF ; if11jef="i2" ;")')cput-cpus,myid,ll,if11jef
   endif
   
   DEALLOCATE(rwk) ; DEALLOCATE(iwk)

   if( if11jef.ne.0 ) then ! terminate iterations; return error flag;
    cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" ; error solving sparse Beltrami linear system; if11jef="i3" ;")')cput-cpus,if11jef
    iflag = -3
    DEALLOCATE(xi)
    if( Ldense ) then ; DEALLOCATE(xio)
    endif
    goto 9999
   !RETURN(mp00aa)
   endif
    
   cput = MPI_WTIME() ; scpu = cput-cpul ! reqd time for sparse linear solver;

   ADDCPU(mp00aa)

   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling up00aa ;")')cput-cpus,myid,ll
   endif

   CALL(up00aa,(NN,xi(1:NN))) ! this assigns At, Az through global; xi is no longer needed;

   if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; called  up00aa ;")')cput-cpus,myid,ll
   endif

   if( Nofe.gt.0 ) then

    if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; calling tr00ab ;")')cput-cpus,myid,ll
    endif
    
    CALL(tr00ab,(Nvol,mns,sfliota(ll,0:1))) ; iotas(0:1)=sfliota(ll,0:1)! At,Az must be valid;  

    if( Wmp00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; called  tr00ab ;")')cput-cpus,myid,ll
    endif

    if( Ldense ) then ; cput = MPI_WTIME() ; write(ounit,1111)cput-cpus,myid,ll,"sparse",sfliota(ll,0:1),scpu
    endif

   endif
   
   RESETCPU

   if( Wmp00aa ) then
    xi(1:NN) = abs(xi(1:NN))
    M1=1 ; M2=NN ; ORDER='D' ; im01caf=0 ; call M01CAF(xi(1:NN),M1,M2,ORDER,im01caf) ! this rearranges xi;
    cput = MPI_WTIME() ; write(ounit,'("mp00aa : "f10.2" :  sparse  xi="999es12.5)')cput-cpus,xi(1:iprint)
   endif

  endif ! end of if( Lsparse );
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item In the case \verb+if(Ldense)+ and \verb+if(Lsparse)+, then the solutions will be compared (for debugging).
!latex In this case, the solution provided by \verb+F11JEF+ will be used for subsequent calculations.
  
  iotaerr(0:1) = zero ; xierr = zero ! used to compare solutions from dense / sparse solvers ;
  
  if( Ldense .and. Lsparse ) then ! can compare solutions;
   
   iotaerr(0:1) = iotas(0:1) - iotad(0:1) ; xierr = sqrt( sum( (xi-xio)**2 ) / NN )
   
   DEALLOCATE(xio) ! this was used to compared the sparse iterative solution with the other solution;
   
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  DEALLOCATE(xi)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Given the Beltrami field, the transform on each of the adjacent interfaces is computed by \verb+tr00ab+, 
!latex Note that \verb+tr00ab+ requires the radial derivatives of the vector potential 
!latex to compute the magnetic field and the interface transforms, 
!latex so \verb+tr00ab+ is only called if \verb+Nofe.gt.0+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item This routine returns an `error'-function, 
!latex given as the difference between the computed transform of the constructed Beltrami field on each of the adjacent interfaces, $\iotabar$, 
!latex and the noble irrationals specified on input
!latex \be \iotabar - \frac{ p_{l} + \gamma p_{r} }{ q_{l} + \gamma q_{r} },
!latex \ee
!latex where $p_{l}\equiv$\verb+pl+, $q_{l}\equiv$\verb+ql+ are supplied on input and $\gamma = ( 1 + \sqrt{5} ) / 2$ is the golden mean.

!latex \item This routine is typically called by \verb+ma02aa+ either directly, 
!latex or indirectly via the NAG routine \verb+C05NBF+ which will adjust the Lagrange multiplier $\mu$ 
!latex and the poloidal flux within each annulus $\psi_{p}$ to set this error function to zero.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Nmudp.eq.1 ) then
   Fmudp(1  ) = sfliota(ll,  1) - (                               pl(ll)+goldmean*pr(ll) ) / (                               ql(ll)+goldmean*qr(ll)  ) ! returned function;
  else
   Fmudp(1:2) = sfliota(ll,0:1) - (/ pl(ll-1)+goldmean*pr(ll-1) , pl(ll)+goldmean*pr(ll)/) / (/ ql(ll-1)+goldmean*qr(ll-1) , ql(ll)+goldmean*qr(ll) /) ! returned function;
  endif
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wma02aa ) then
   cput = MPI_WTIME() 
   if( Nmudp.eq.1 ) write(ounit,1000)cput-cpus,myid,ll,imupf,mu(ll),dpflux(ll),sfliota(ll,0:1),iotaerr(0:1),xierr,Fmudp(1:Nmudp),dcpu,scpu
   if( Nmudp.eq.2 ) write(ounit,1001)cput-cpus,myid,ll,imupf,mu(ll),dpflux(ll),sfliota(ll,0:1),iotaerr(0:1),xierr,Fmudp(1:Nmudp),dcpu,scpu
1000 format("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; its="i3" ; (mu,pf)=("2f20.16" ) ; sfliota="2f20.16" ; ("2es8.1" ) ; xi err="es9.1" ; F=" es10.2,10x" ; time="2f10.6" ;")
1001 format("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; its="i3" ; (mu,pf)=("2f20.16" ) ; sfliota="2f20.16" ; ("2es8.1" ) ; xi err="es9.1" ; F="2es10.2" ; time="2f10.6" ;")
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if(         imupf            .eq. mupfits ) iflag = -1 ! terminate iterations; user requested termination; see mupfits description in global;
  if( sum(abs(Fmudp(1:Nmudp))) .lt. mupftol ) iflag = -2 ! terminate iterations; user requested termination; see mupftol description in global;

  imupf = imupf + 1 ! iteration counter; initialized in ma02aa;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(mp00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

1111 format("mp00aa : "f10.2" : myid="i3" ; ll="i3" ; "a6" interface transforms ="2es23.15" ; time="f10.6" ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine mp00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
